part of '../documentation_repository.dart';

Section _border() => SectionBuilder()
    .add(
      id: "borderWidth",
      title: "Border Width",
      description: "Utilities for controlling the width of an element's borders.",
    )
    .addPart(
      title: "All sides",
      text: [
        Section.textSpan(text: "Use the "),
        Section.textSpanColored(text: "`border-none`"),
        Section.textSpan(text: ","),
        Section.textSpanColored(text: "`border-[num]`"),
        Section.textSpan(text: ","),
        Section.textSpanColored(text: "`border-3`"),
        Section.textSpan(text: ","),
        Section.textSpanColored(text: "`border-5`"),
        Section.textSpan(text: ", or "),
        Section.textSpanColored(text: "`border-10`"),
        Section.textSpan(text: " utilities to set the border width for all sides of an element."),
      ],
      builder: (c) => _BorderWidthAllSides(),
    )
    .addPart(
      title: "Individual sides",
      text: [
        Section.textSpan(text: "Use the "),
        Section.textSpanColored(text: "`border-{side}-none`"),
        Section.textSpan(text: ","),
        Section.textSpanColored(text: "`border-{side}-[num]`"),
        Section.textSpan(text: ","),
        Section.textSpanColored(text: "`border-{side}-3`"),
        Section.textSpan(text: ","),
        Section.textSpanColored(text: "`border-{side}-5`"),
        Section.textSpan(text: ", or "),
        Section.textSpanColored(text: "`border-{side}-10`"),
        Section.textSpan(text: " utilities to set the border width for one side of an element."),
      ],
      builder: (c) => _BorderWidthIndividualSides(),
    )
    .build();

class _BorderWidthAllSides extends StatelessWidget {
  @override
  Widget build(BuildContext context) => _SectionPartWidget(
        topWidget: Div(
          style: "flex p-50 space-x-12",
          children: [
            _Box(style: "border-none", title: "border none"),
            _Box(style: "border-1", title: "border 1"),
            _Box(style: "border-2", title: "border 2"),
            _Box(style: "border-4", title: "border 4"),
            _Box(style: "border-8", title: "border 8"),
          ],
        ),
        bottomWidget: Div(
          color: TWColors.fromHex("#21233f"),
          style: "${_$codeBoxStyle} text-17 leading-20 tracking-34 object-center",
          children: [
            CodeViewer(
              topics: Ranges.forPlugin("borderWidth"),
              code:
                  """Box(style: 'border-none', title: 'border none'),\nBox(style: 'border-1', title: 'border 1'),\nBox(style: 'border-2', title: 'border 2'),\nBox(style: 'border-4', title: 'border 4'),\nBox(style: 'border-8', title: 'border 8'),""",
              // width: 500,
            )
          ],
        ),
      );
}

class _BorderWidthIndividualSides extends StatelessWidget {
  @override
  Widget build(BuildContext context) => _SectionPartWidget(
        topWidget: Div(
          style: "flex p-50 space-x-12",
          children: [
            _Box(style: "border-none", title: "border none"),
            _Box(style: "border-t-2", title: "border top"),
            _Box(style: "border-r-2", title: "border right"),
            _Box(style: "border-b-2", title: "border bottom"),
            _Box(style: "border-l-2", title: "border left"),
          ],
        ),
        bottomWidget: Div(
          color: TWColors.fromHex("#21233f"),
          style: "${_$codeBoxStyle} text-17 leading-20 tracking-34 object-center",
          children: [
            CodeViewer(
              topics: Ranges.forPlugin("borderWidth"),
              code:
                  """Box(style: 'border-none', title: 'border none'),\nBox(style: 'border-1', title: 'border 1'),\nBox(style: 'border-2', title: 'border 2'),\nBox(style: 'border-4', title: 'border 4'),\nBox(style: 'border-8', title: 'border 8'),""",
              // width: 500,
            )
          ],
        ),
      );
}
