part of '../documentation_repository.dart';

Section _borderRadius() => SectionBuilder()
    .add(
      id: "borderRadius",
      title: "Border Radius",
      description: "Utilities for controlling the border radius of an element.",
    )
    .addPart(
      title: "Rounded corners",
      text: [
        Section.textSpan(text: "Use utilities like "),
        Section.textSpanColored(text: "`rounded-[num]`"),
        Section.textSpan(text: ","),
        Section.textSpanColored(text: "`rounded-sm`"),
        Section.textSpan(text: ","),
        Section.textSpanColored(text: "`rounded-lg`"),

        Section.textSpan(text: " to apply different border radius sizes to an element."),
      ],
      builder: (c) => _BorderRadiusAllSides(),
    )
    .addPart(
      title: "Pills and circles",
      text: [
        Section.textSpan(text: "Use the "),
        Section.textSpanColored(text: "`rounded-full`"),
        Section.textSpan(text: " utility to create pills and circles."),
      ],
      builder: (c) => _BorderRadiusIndividualSides(),
    )
    .build();

class _BorderRadiusAllSides extends StatelessWidget {
  @override
  Widget build(BuildContext context) => _SectionPartWidget(
        topWidget: Div(
          style: "flex p-50 space-x-12",
          children: [
            _Box(style: "rounded-none", title: "border none"),
            _Box(style: "rounded-full", title: "border 1"),
            _Box(style: "rounded-10", title: "border 2"),
            _Box(style: "rounded-lg", title: "border 4"),
          ],
        ),
        bottomWidget: Div(
          color: TWColors.fromHex("#21233f"),
          style: "${_$codeBoxStyle} text-17 leading-20 tracking-34 object-center",
          children: [
            CodeViewer(
              topics: Ranges.forPlugin("borderWidth"),
              code:
                  """Box(style: 'border-none', title: 'border none'),\nBox(style: 'border-1', title: 'border 1'),\nBox(style: 'border-2', title: 'border 2'),\nBox(style: 'border-4', title: 'border 4'),\nBox(style: 'border-8', title: 'border 8'),""",
              // width: 500,
            )
          ],
        ),
      );
}

class _BorderRadiusIndividualSides extends StatelessWidget {
  @override
  Widget build(BuildContext context) => _SectionPartWidget(
        topWidget: Div(
          style: "flex p-50 space-x-12",
          children: [
            _Box(style: "rounded-tr-30 rounded-bl-xl", title: "border none"),
            _Box(style: "rounded-10", title: "border top"),
            _Box(style: "rounded-xl", title: "border right"),
            _Box(style: "rounded-3xl", title: "border bottom"),
          ],
        ),
        bottomWidget: Div(
          color: TWColors.fromHex("#21233f"),
          style: "${_$codeBoxStyle} text-17 leading-20 tracking-34 object-center",
          children: [
            CodeViewer(
              topics: Ranges.forPlugin("borderWidth"),
              code:
                  """Box(style: 'border-none', title: 'border none'),\nBox(style: 'border-1', title: 'border 1'),\nBox(style: 'border-2', title: 'border 2'),\nBox(style: 'border-4', title: 'border 4'),\nBox(style: 'border-8', title: 'border 8'),""",
              // width: 500,
            )
          ],
        ),
      );
}
