part of 'documentation_pdf.dart';

typedef Future<Uint8List> PDFFactory(PdfPageFormat format);

List<pw.Widget> sectionsToWidgets(List<Section> sections) {
  var widgets = List<pw.Widget>();
  for (var section in sections) {
    var _w = section.asPdfWidgets();
    widgets.addAll(_w);
  }
  // print(["Ovo je sekcija_", widgets]);

  return widgets;
}

PDFFactory pdfFromSections(List<Section> sections) {
  return (PdfPageFormat format) async {
    final doc = pw.Document(title: 'My Résumé', author: 'David PHAM-VAN');

    final pageTheme = await _myPageTheme(format);

    doc.addPage(
      pw.MultiPage(
        pageTheme: pageTheme,
        build: (pw.Context context) => [
          pw.Partitions(
            children: [
              pw.Partition(
                child: pw.Column(
                  crossAxisAlignment: pw.CrossAxisAlignment.start,
                  children: <pw.Widget>[
                    ...sectionsToWidgets(sections),
                  ],
                ),
              ),
            ],
          ),
        ],
      ),
    );
    return doc.save();
  };
}

Future<pw.PageTheme> _myPageTheme(PdfPageFormat format) async {
  final bgShape = await rootBundle.loadString('assets/pdf/resume.svg');

  format = format.applyMargin(
      left: 2.0 * PdfPageFormat.cm,
      top: 4.0 * PdfPageFormat.cm,
      right: 2.0 * PdfPageFormat.cm,
      bottom: 2.0 * PdfPageFormat.cm);
  return pw.PageTheme(
    pageFormat: format,
    theme: pw.ThemeData.withFont(
      base: pw.Font.ttf(await rootBundle.load('assets/pdf/open-sans.ttf')),
      bold: pw.Font.ttf(await rootBundle.load('assets/pdf/open-sans-bold.ttf')),
      icons: pw.Font.ttf(await rootBundle.load('assets/pdf/material.ttf')),
    ),
    buildBackground: (pw.Context context) {
      return pw.FullPage(
        ignoreMargins: true,
        child: pw.Stack(
          children: [
            pw.Positioned(
              child: pw.SvgImage(svg: bgShape),
              left: 0,
              top: 0,
            ),
            pw.Positioned(
              child: pw.Transform.rotate(angle: pi, child: pw.SvgImage(svg: bgShape)),
              right: 0,
              bottom: 0,
            ),
          ],
        ),
      );
    },
  );
}

const PdfColor _green = PdfColor.fromInt(0xff9ce5d0);
const PdfColor _lightGreen = PdfColor.fromInt(0xffcdf1e7);
const _sep = 120.0;

class DBlockPdf extends pw.StatelessWidget {
  DBlockPdf({
    this.title,
    this.description,
    this.icon,
  });

  final String title;
  final String description;

  final pw.IconData icon;

  @override
  pw.Widget build(pw.Context context) {
    return pw.Column(crossAxisAlignment: pw.CrossAxisAlignment.start, children: <pw.Widget>[
      pw.Row(crossAxisAlignment: pw.CrossAxisAlignment.start, children: <pw.Widget>[
        pw.Container(
          width: 6,
          height: 6,
          margin: const pw.EdgeInsets.only(top: 2.5, left: 2, right: 5),
          decoration: const pw.BoxDecoration(color: _green, shape: pw.BoxShape.circle),
        ),
        pw.Text(title, style: pw.Theme.of(context).defaultTextStyle.copyWith(fontWeight: pw.FontWeight.bold)),
        pw.Spacer(),
        if (icon != null) pw.Icon(icon, color: _lightGreen, size: 18),
      ]),
      pw.Container(
        decoration: const pw.BoxDecoration(border: pw.Border(left: pw.BorderSide(color: _green, width: 2))),
        padding: const pw.EdgeInsets.only(left: 10, top: 5, bottom: 5),
        margin: const pw.EdgeInsets.only(left: 5),
        child: pw.Column(crossAxisAlignment: pw.CrossAxisAlignment.start, children: <pw.Widget>[
          pw.Text(description ?? ""),
        ]),
      ),
    ]);
  }
}
