import 'dart:typed_data';

import 'package:equatable/equatable.dart';
import 'package:example/documentation/core/printable.dart';
import 'package:flutter/material.dart';
import 'package:pdf/pdf.dart';
import 'package:pdf/widgets.dart' as pw;

class Section extends Printable with EquatableMixin {
  final String id;
  final String title;
  final String description;
  List<SectionPart> parts = [];
  List<pw.TextSpan> text;

  Section(
    this.id, {
    this.title,
    this.description,
    this.text = const [],
  });

  @override
  List<Object> get props => [id];

  @override
  List<pw.Widget> asPdfWidgets() {
    var widgets = List<pw.Widget>();
    var text = List<pw.TextSpan>();
    widgets.add(pw.Text(title, style: titleStyle()));
    widgets.add(dividerSm());
    if (description != null) {
      widgets.add(pw.Text(description, style: textStyle()));
    }

    widgets.add(
      pw.RichText(
        text: pw.TextSpan(
          children: [...text, ...this.text],
        ),
      ),
    );

    widgets.add(divider());

    if (parts.isNotEmpty) {
      widgets.addAll(
        parts.map((e) => e.asPdfWidgets()).reduce((a, b) => [...a, ...b]),
      );
    }
    widgets.add(divider());
    return widgets;
  }

  @override
  Widget asWidget() => Column(
        children: parts.map((e) => e.asWidget()).toList(growable: false),
      );

  static PdfColor textColor({PdfColor color}) => color ?? PdfColors.purple;

  static pw.TextStyle titleStyle({PdfColor color}) =>
      pw.TextStyle(color: color, fontSize: 20, fontWeight: pw.FontWeight.bold);

  static TextStyle wTitleStyle({Color color}) => TextStyle(color: color, fontSize: 20, fontWeight: FontWeight.bold);

  static pw.TextStyle subTitleStyle({PdfColor color}) =>
      pw.TextStyle(color: color, fontSize: 16, fontWeight: pw.FontWeight.bold);

  static TextStyle wSubTitleStyle({Color color}) => TextStyle(color: color, fontSize: 16, fontWeight: FontWeight.bold);

  static pw.TextStyle textStyle({PdfColor color}) => pw.TextStyle(color: color, fontSize: 12);

  static TextStyle wTextStyle({Color color}) => TextStyle(color: color, fontSize: 12);

  static pw.TextSpan textSpan({String text}) => pw.TextSpan(text: text, style: textStyle());

  static pw.TextSpan textSpanColored({PdfColor color, String text}) =>
      pw.TextSpan(text: text, style: textStyle(color: textColor(color: color)));

  static pw.SizedBox dividerXxl({double height}) => pw.SizedBox(height: height ?? 30);

  static pw.SizedBox dividerXl({double height}) => pw.SizedBox(height: height ?? 20);

  static pw.SizedBox dividerLg({double height}) => pw.SizedBox(height: height ?? 15);

  static pw.SizedBox divider({double height}) => pw.SizedBox(height: height ?? 10);

  static pw.SizedBox dividerSm({double height}) => pw.SizedBox(height: height ?? 5);
}

class SectionPart extends Printable {
  final int index;

  final String title;

  final String description;

  //Ovo je kljucni elemenat kao slika
  final WidgetBuilder builder;

  Uint8List imageBytes;
  List<pw.TextSpan> text;

  SectionPart({
    this.index = 0,
    this.title,
    this.builder,
    this.description,
    this.imageBytes,
    this.text = const [],
  });

  @override
  List<pw.Widget> asPdfWidgets() {
    var widgets = <pw.Widget>[];
    var text = <pw.TextSpan>[];
    widgets.add(pw.Text(title, style: Section.subTitleStyle()));
    widgets.add(Section.divider());
    if (description != null) {
      text.add(pw.TextSpan(text: description));
    }

    widgets.add(
      pw.RichText(
        text: pw.TextSpan(
          children: [...text, ...this.text],
        ),
      ),
    );
    widgets.add(Section.divider());

    if (imageBytes != null) {
      widgets.add(pw.Image.provider(
        pw.MemoryImage(imageBytes),
      ));
      widgets.add(Section.divider());
    }
    widgets.add(Section.dividerXl());
    return widgets;
  }

  GenerateImageFunction action;

  @override
  Widget asWidget() {
    var widgets = <Widget>[];
    var text = <InlineSpan>[];

    var _text = this
        .text
        .map(
          (e) => TextSpan(text: e.text, style: TextStyle(color: Colors.black)),
        )
        .toList(growable: false);
    widgets.add(Text(title, style: Section.wSubTitleStyle()));
    if (description != null) {}

    widgets.add(
      RichText(
        text: TextSpan(
          children: [...text, ..._text],
        ),
      ),
    );

    // widgets.add(Builder(builder: (c) => builder(c)));

    return Column(
      mainAxisSize: MainAxisSize.min,
      children: widgets,
    );
  }
}

typedef Future<Uint8List> GenerateImageFunction();
