import 'package:daggerito/daggerito.dart';
import 'package:example/documentation/documentation_pdf.dart';
import 'package:example/documentation/generate_content_page.dart';
import 'package:example/documentation/generate_images_page.dart';
import 'package:example/documentation/view_model/section_view_model.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:kohana/kohana.dart';
import 'package:rxdart/rxdart.dart';
import 'package:apply/apply.dart';

// ignore: must_be_immutable
class PluginDocumentationPage extends StatefulView<_Fields> {
  final Component component;

  PluginDocumentationPage(this.component);

  @override
  Widget buildWidget(BuildContext context) => Scaffold(
        appBar: AppBar(
          actions: [
            Button(
              style: "text-white",
              title: "Submit",
              onPressed: $.model.captureImages,
            )
          ],
        ),
        body: buildTabBarView(context),
        bottomNavigationBar: buildTabBarItems(context),
      );

  Widget buildTabBarView(BuildContext context) => TabBarView(
        controller: $._tabController,
        children: [
          Div(
            style: "mx-10",
            child: GenerateContentPage(model: $.model),
          ),
          GenerateImagesPage(model: $.model),
          DocumentationPdf(model: $.model),
        ],
      );

  Widget buildTabBarItems(BuildContext context) => Observer(
        stream: $.currentTab.stream,
        onSuccess: (c, index) => BottomNavigationBar(
          currentIndex: index,
          onTap: $.currentTab.add,
          items: $.tabs,
        ),
      );

  @override
  FieldsFactory<_Fields> setup() {
    var tickerProvider = useSingleTickerProvider();
    onMounted(() async {
      // print("On mounted");
      disposeBag.addAll([$.currentTab.listen((value) => $._tabController.index = value)]);
      $._tabController.addListener(() => $.currentTab.add($._tabController.index));
    });
    return () => _Fields(
          component(),
          _createTabs(),
          TabController(length: 3, vsync: tickerProvider),
        );
  }
}

class _Fields extends HasModelField<SectionViewModel> {
  final List<BottomNavigationBarItem> tabs;
  final TabController _tabController;
  final currentTab = BehaviorSubject.seeded(0);
  @override
  final SectionViewModel model;

  _Fields(this.model, this.tabs, this._tabController);

  @override
  void dispose() {
    currentTab.close();
    _tabController.dispose();
    super.dispose();
  }
}

List<BottomNavigationBarItem> _createTabs() => [
  BottomNavigationBarItem(
    label: "Content",
    icon: Icon(Icons.note_add),
  ),
      BottomNavigationBarItem(
        label: "Documentation",
        icon: Icon(Icons.wallpaper_rounded),
      ),
      BottomNavigationBarItem(
        label: "Pdf",
        icon: Icon(Icons.tab),
      ),
    ];
