import 'dart:async';
import 'dart:typed_data';

import 'package:example/db/documentation_repository.dart';
import 'package:example/documentation/model/section.dart';
import 'package:example/utils.dart';
import 'package:example/view_model/capture_image_view_model.dart';
import 'package:example/view_model/view_model.dart';
import 'package:example/widgets/widget_to_image.dart';
import 'package:flutter/material.dart';
import 'package:kohana/kohana.dart';
import 'package:rxdart/rxdart.dart';

class SectionViewModel extends ViewModel {
  final DocumentationRepository repository;

  SectionViewModel(this.repository);

  Map<KeyWithDescription, Uint8List> _images = {};
  Set<Section> _sections = {};
  final _widgets = BehaviorSubject<List<WidgetBuilder>>();

  Stream<List<WidgetBuilder>> get widgets => _widgets.stream;

  final _imagesSubject = BehaviorSubject<Map<KeyWithDescription, Uint8List>>.seeded({});

  Stream<List<Section>> get sections {
    return _imagesSubject.stream.map((event) {
      var imagesWithKeys = event.entries;
      for (var imageWithKey in imagesWithKeys) {
        var keyWithDescription = imageWithKey.key;
        Uint8List imageBytes = imageWithKey.value;
        var id = keyWithDescription.asId;
        var index = keyWithDescription.asIndex;
        _sections.setImage(id, index, imageBytes);
      }
      return _sections.toList(growable: false);
    });
  }

  FutureOr start() async {
    print("Ovo je start");
    var sectionsStream = repository.allSections().asStream();
    // var timer = Rx.timer(1, Duration(milliseconds: 2000));
    disposeBag.addAll([
      // sectionsStream.doOnData(setSections).switchMap((_) => timer).listen((_) => captureImages()),
      // sectionsStream.doOnData(setSections).switchMap((_) => timer).listen((_) => handleImages()),
      sectionsStream.listen(setSections),
    ]);
  }

  void addKey(Key key, String id) {
    var keyWithDescription = KeyWithDescription(key, id);
    _images.putIfAbsent(keyWithDescription, () => null);
  }

  void captureImages() async {
    Map<KeyWithDescription, Uint8List> newImages = {};
    for (var entry in _images.entries) {
      var keyWithDescription = entry.key;
      final imageBytes = await Utils.capture(keyWithDescription.key);
      newImages[keyWithDescription] = imageBytes;
    }
    _imagesSubject.add(newImages);
    _images = {}..addAll(newImages);
    print("Ovo je capture images");
  }

  void setSections(List<Section> sections) {
    List<WidgetBuilder> wbs = [];
    _sections.addAll(sections);
    for (var section in sections) {
      for (var part in section.parts) {
        var builder = part.builder;
        var action = (key) async {
          part.imageBytes = await Utils.capture(key);
        };
        wbs.add(
          (context) => KeepAliveViewWrapper(
            child: WidgetToImage2(
              part,
              builder: (key) {
                addKey(key, "${section.id}.${part.index}");
                return builder(context);
              },
              action: action,
            ),
          ),
        );
      }
    }
    _widgets.add(wbs);
  }

  @override
  FutureOr dispose() async {
    await _imagesSubject.close();
    await _widgets.close();
    await super.dispose();
  }
}

extension on KeyWithDescription {
  String get asId => this.id.split(".")[0];

  int get asIndex => int.parse(this.id.split(".")[1]);
}

extension on Set<Section> {
  void setImage(String id, int index, Uint8List imageBytes) {
    var section = this.firstWhere((element) => element.id == id);
    var sectionPart = section.parts.firstWhere((element) => element.index == index);
    sectionPart.imageBytes = imageBytes;
  }
}
