import 'dart:async';
import 'dart:typed_data';

import 'package:equatable/equatable.dart';
import 'package:example/documentation/model/section.dart';
import 'package:example/utils.dart';
import 'package:flutter/material.dart';
import 'package:kohana/kohana.dart';
import 'package:rxdart/rxdart.dart';

class CaptureImagesViewModel implements LifecycleAware {
  Map<KeyWithDescription, Uint8List> _images = {};
  final _imagesSubject = BehaviorSubject<Map<KeyWithDescription, Uint8List>>.seeded({});
  final _counter = BehaviorSubject<int>.seeded(0);

  Stream<int> get counter => Stream.periodic(Duration(milliseconds: 1000), (i) => i);

  Stream<List<Uint8List>> get images =>
      _imagesSubject.stream.map((event) => event.entries.map((e) => e.value).toList(growable: false));

  void addKey(KeyWithDescription keyWithDescription) {
    _images[keyWithDescription] = null;
  }

  void captureImages() async {
    Map<KeyWithDescription, Uint8List> newImages = {};
    for (var entry in _images.entries) {
      final imageBytes = await Utils.capture(entry.key.key);
      newImages[entry.key] = imageBytes;
    }
    _imagesSubject.add(newImages);
    _images = {}..addAll(newImages);
  }

  FutureOr start() async {}

  @override
  FutureOr dispose() {
    _imagesSubject.close();
    _counter.close();
  }

  CaptureImagesViewModel() {
    // print("Kreira se view model");
  }
}


class KeyWithDescription with EquatableMixin{
  final Key key;
  final String id;

  KeyWithDescription(this.key, this.id);

  @override
  List<Object> get props => [key, id];
}
