import 'package:flutter/material.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/plugins_service.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/plugins/font-weight.dart';
import 'package:apply/plugins/font_family.dart';
import 'package:apply/plugins/font_size.dart';
import 'package:apply/plugins/height.dart';
import 'package:apply/plugins/letter_spacing.dart';
import 'package:apply/plugins/line_height.dart';
import 'package:apply/plugins/text_align.dart';
import 'package:apply/plugins/text_color.dart';
import 'package:apply/plugins/text_decoration.dart';
import 'package:apply/plugins/text_opacity.dart';
import 'package:apply/plugins/width.dart';

extension ExtGetPluginFromId<T extends Plugin> on Set<T> {
  T getPluginById(String id) {
    var plugin = PluginsService.findById(id);
    if(this == null){
      return null;
    }
    if (this.contains(plugin)) {
      return this.where((e) => e.id == id).first;
    }

    return null;
  }

  T getPluginByIdOrDefault(String id) => this.getPluginById(id) ?? PluginsService.findById(id);

  bool removeById(String id) => this.remove(PluginsService.findById(id));

  List<Plugin> getInheritedPlugins() {
    var typesToInherit = [
      FontWeightP,
      FontFamily,
      FontSize,
      LineHeight,
      TextAlignP,
      TextColorP,
      TextOpacity,
      TextDecorationP,
      LetterSpacing,
    ];
    return this.where((element) => typesToInherit.contains(element.runtimeType)).toList(growable: false);
  }
}

extension ExtFlex on Flex {
  Flex copyWith({
    List<Widget> children = const [],
    MainAxisAlignment mainAxisAlignment = MainAxisAlignment.start,
    CrossAxisAlignment crossAxisAlignment = CrossAxisAlignment.center,
  }) =>
      direction == Axis.horizontal
          ? Row(
              children: children,
              mainAxisAlignment: mainAxisAlignment,
              mainAxisSize: this.mainAxisSize,
              crossAxisAlignment: crossAxisAlignment,
            )
          : Column(
              children: children,
              mainAxisAlignment: mainAxisAlignment,
              mainAxisSize: this.mainAxisSize,
              crossAxisAlignment: crossAxisAlignment,
            );
}

typedef Widget ComposableWidget(WidgetBuilder child);

class Pair<E, F> {
  final E first;
  final F last;

  Pair(this.first, this.last);

  @override
  String toString() => 'Pair{first: $first, last: $last}';
}

double radiansFromDegrees(double degrees) => degrees * 0.0174533;

extension HasDiv on Widget {
  bool hasDiv(BuildContext context) {
    if (this is Div) {
      return true;
    } else {
      var _widget = (this as StatelessWidget).build(context);
      return _widget is Div;
      // return widget != null;
    }
  }

  Div setOwnPlugins(BuildContext context) {
    assert(hasDiv(context));
    Div widget;
    if (this is Div) {
      widget = this;
    } else {
      var _widget = (this as StatelessWidget).build(context);
      if(_widget is Div){
        widget = _widget;
      }
      if (widget == null) {
        return null;
      }
    }

    widget.setAttributes(context);
    widget.ownPlugins = widget.attributes.getAppliedPlugins();
    var options = widget.divOptions;
    double width = options.width;
    double height = options.height;
    var plugins = widget.ownPlugins;

    Width widthPlugin = plugins.getPluginById("width");
    Height heightPlugin = plugins.getPluginById("height");
    if (widthPlugin != null) {
      width = widthPlugin.getWidth();
    }

    if (heightPlugin != null) {
      height = heightPlugin.getHeight();
    }

    widget.widthAndHeightPair = Pair(width, height);
    return widget;
  }
}
