library simple_layout;

import 'package:flutter/material.dart';
import 'package:apply/core/framework.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/plugins/font-weight.dart';
import 'package:apply/plugins/font_family.dart';
import 'package:apply/plugins/font_size.dart';
import 'package:apply/plugins/letter_spacing.dart';
import 'package:apply/plugins/line_height.dart';
import 'package:apply/plugins/text_align.dart';
import 'package:apply/plugins/text_color.dart';
import 'package:apply/plugins/text_decoration.dart';
import 'package:apply/plugins/text_opacity.dart';

class P extends StatelessWidget {
  final String text;
  final String style;

  P(
    this.text, {
    this.style = "",
  });

  @override
  Widget build(BuildContext context) => Div(
        style: style,
        child: _P(text),
      );

  static Pair<TextAlign, TextStyle> getTextStyle(BuildContext context) => _getTextStyle(context);
}

class _P extends StatelessWidget {
  final String text;

  _P(this.text);

  @override
  Widget build(BuildContext context) {
    var textStyle = _getTextStyle(context);

    return Text(
      text,
      style: textStyle.last,
      textAlign: textStyle.first,
    );
  }
}

Pair<TextAlign, TextStyle> _getTextStyle(BuildContext context) {
  var enclosingDiv = context.findAncestorWidgetOfExactType<Div>();
  if (enclosingDiv == null) {
    return null;
  }
  TextDecoration textDecoration;

  TextDecorationP textDecorationP = enclosingDiv.plugins.getPluginById("textDecoration");
  if (textDecorationP != null) {
    textDecoration = textDecorationP.getTextDecoration();
  }

  double fontSize;
  double lineHeight = 1.5;
  double letterSpacing = 0;
  String fontFamily;
  FontWeight fontWeight;
  Color textColor;
  TextAlign textAlign;
  TextStyle textStyle = TextStyle(
    letterSpacing: letterSpacing,
    height: lineHeight,
    decoration: textDecoration,
  );

  FontSize fontSizePlugin = enclosingDiv.plugins.getPluginByIdOrDefault("fontSize");
  FontFamily fontFamilyPlugin = enclosingDiv.plugins.getPluginById("fontFamily");
  FontWeightP fontWeightPlugin = enclosingDiv.plugins.getPluginById("fontWeight");
  LineHeight lineHeightPlugin = enclosingDiv.plugins.getPluginById("lineHeight");
  LetterSpacing letterSpacingPlugin = enclosingDiv.plugins.getPluginById("letterSpacing");
  TextAlignP textAlignPlugin = enclosingDiv.plugins.getPluginById("textAlign");
  TextColorP textColorPlugin = enclosingDiv.plugins.getPluginById("textColor");
  TextOpacity textOpacityPlugin = enclosingDiv.plugins.getPluginById("textOpacity");

  if (fontSizePlugin != null) {
    fontSize = fontSizePlugin.fontSize();
    textStyle = textStyle.copyWith(fontSize: fontSize);
  }

  if (lineHeightPlugin != null) {
    lineHeight = lineHeightPlugin.getLineHeight(fontSize);
    textStyle = textStyle.copyWith(height: lineHeight);
  }

  if (letterSpacingPlugin != null) {
    letterSpacing = letterSpacingPlugin.getLetterSpacing(fontSize);
    textStyle = textStyle.copyWith(letterSpacing: letterSpacing);
  }

  if (fontFamilyPlugin != null) {
    fontFamily = fontFamilyPlugin.fontFamily();
    textStyle = textStyle.copyWith(fontFamily: fontFamily);
  }

  if (fontWeightPlugin != null) {
    fontWeight = fontWeightPlugin.getFontWeight();
    textStyle = textStyle.copyWith(fontWeight: fontWeight);
  }

  if (textColorPlugin != null) {
    textColor = textColorPlugin.getColor(enclosingDiv);
    textStyle = textStyle.copyWith(color: textColor);
  }

  if (textOpacityPlugin != null) {
    if(textColor == null){
      textColor = Theme.of(context).textTheme.bodyText1.color;
    }
    textColor = textColor.withOpacity(textOpacityPlugin.getColorOpacity());
    textStyle = textStyle.copyWith(color: textColor);
  }

  if (textAlignPlugin != null) {
    textAlign = textAlignPlugin.getTextAlign();
  }

  return Pair(textAlign, textStyle);
}
