const watch = require("node-watch");
const dependencies = require("./dependencies");

const { readFile, writeFile } = require("fs");
const path = require("path");

function handleData(fileName, html) {
  console.log("Sada ima novih podataka");

  let find = html.filter((node) => {
    return node.name === "template";
  });

  if (find.length > 0) {
    let element = find[0];
    let id;
    if (element.attribs["id"] === undefined) {
      id = "main";
    } else {
      id = element.attribs["id"];
    }
    let finalText = new ElementToDartClass().handle(element);
    console.dir(finalText);
    // replace(`//\n${finalText}\n//`);
    replaceCodeWith(fileName, { id: id, content: `{{\n${finalText}\n//}}` });
    // replace(finalText);
  }
}

watch(
  "./",
  {
    recursive: true,
    filter: /\.dart\.html$/,
  },
  function (evt, name) {
    console.log("%s changed.", name);

    // const filePath = path.join(__dirname, name);
    const filePath = path.join("./", name);
    readFile(name, "utf8", function (err, data) {
      if (err) {
        return console.log(err);
      }
      new HtmlParser(filePath, handleData).process(data);
    });
  }
);

class HtmlParser {
  constructor(forFile, domConsumer) {
    this.forFile = forFile;
    this.domConsumer = domConsumer;
    this.handle = (error, dom) => {
      if (error) {
        // Handle error
      } else {
        // Parsing completed, do something
        this.domConsumer(forFile, dom);
      }
    };
    this.handler = new dependencies.DomHandler.DomHandler(this.handle);
    this.parser = new dependencies.Parser(this.handler);
  }

  process(rawHtml) {
    this.parser.write(rawHtml);
    this.parser.end();
  }
}

class ElementToDartClass {
  constructor() {
    this.handle = (element) => {
      let entry = ``;
      let before = ``;
      let after = ``;
      const className = processStringToDartClassName(element.name);
      let children = element.children;
      if (element.type === "text") {
        // let data = element.data.replace(/[^0-9a-z]/gi, '');
        let data = element.data.trim();
        if (data.length === 0) {
          return "";
        } else {
          let parent = processStringToDartClassName(element.parent.name);
          if (["P", "Text"].includes(parent)) {
            return `"""\n${data}\n"""`;
          } else {
            return `P("""\n${data}\n"""),`;
          }
        }
      }
      if (!className) {
        return "";
      }
      if (className === "Template") {
        before += ``;
      } else {
        before += `${className}(\n`;
      }
      const attribs = element.attribs;
      for (const [key, value] of Object.entries(attribs)) {
        if (key === "arg1") {
          before += `${value},\n`;
        } else if (key === "class") {
          before += `style: "${value}",\n`;
        } else {
          before += `${key}: ${value},\n`;
        }
      }
      for (let i = 0; i < children.length; i++) {
        let tag = children[i];
        if (tag.type === "text") {
          // let data = tag.data.replace(/[^0-9a-z]/gi, '');
          let data = tag.data.trim();
          if (data.length === 0) {
            children.splice(i, 1);
          }
        }
      }
      for (let i = 0; i < children.length; i++) {
        let tag = children[i];
        entry += this.handle(tag);
      }
      if (className === "Template") {
        after += ``;
      } else {
        after += `),\n`;
      }

      return before + this.buildBody(className, children)(entry) + after;
    };
    this.buildBody = (className, children) => {
      if (children === undefined) {
        children = [];
      }
      const doNothingBuilder = (c) => c;
      const childrenBuilder = (c) => `children: [
           ${c}
             ],`;
      const childBuilder = (c) => `child:
           ${c}
             `;
      if (children.length === 0) {
        return doNothingBuilder;
      }
      if (children.length > 1) {
        return childrenBuilder;
      }
      if (
        ["Column", "Row", "Stack", "Flex", "Div", "Template"].includes(
          className
        )
      ) {
        return childrenBuilder;
      } else if (["P", "Text"].includes(className)) {
        return doNothingBuilder;
      } else {
        return childBuilder;
      }
    };
  }
}

function processStringToDartClassName(input) {
  if (input === undefined) return input;
  return input.replace(/(^|[\s-])\S/g, function (match) {
    return match.toUpperCase().replace("-", "");
  });
}

function replaceCodeWith(fileName, content) {
  // let someFile = "./" + fileName;

  let someFile = fileName;
  let withContent = content.content;
  let id = content.id;
  someFile = someFile.replace(/(\.dart.\html)/g, ".dart");
  readFile(someFile, "utf8", function (err, data) {
    if (err) {
      return console.log(err);
    }
    const result = data.replace(/({{)([\s\S]*?)(}})/g, withContent); // + id
    // const result = data.replace(/Template\(([^()]|(\?R))*\)/g, withContent);

    writeFile(someFile, result, "utf8", function (err) {
      if (err) return console.log(err);
    });
  });
}
